using System;
using System.IO;
using System.Threading.Tasks;
using ABI.CCK.Interface;
using ABI.CCK.Scripts.Editor;
using UnityEditor;
using UnityEngine;
using UnityEngine.Networking;
using UnityEngine.PlayerLoop;
using Abi.Newtonsoft.Json;

namespace ABI.MODS.Editor
{
    [InitializeOnLoad]
    public static class CVR_CCKAutoUpdater
    {
        static CVR_CCKAutoUpdater()
        {
            CheckForUpdates();
        }

        static async void CheckForUpdates()
        {
            using (UnityWebRequest uwr = UnityWebRequest.Get("https://gateway.abi.network/v1/IContentCreation/CurrentCvrCreationKitInfo"))
            {
                CCKVersionCurrent c = JsonConvert.DeserializeObject<CCKVersionCurrent>(uwr.downloadHandler.text);

                if (c == null) return;
                
                if (c.BuildID > CCK_BuildManagerWindow.BuildID)
                    if (EditorUtility.DisplayDialog(
                        "Update ChilloutVR Content Creation Kit",
                        $"We have found an update for the ChilloutVR Content Creation Kit (Newest version: {c.BuildVersion} | Yours: {CCK_BuildManagerWindow.Version}",
                        "Update Now", "Ignore")) await DownloadAndImportUpdate(c.DownloadLink);
            }
        }

        static async Task DownloadAndImportUpdate(string url)
        {
            using (UnityWebRequest uwr = UnityWebRequest.Get(url))
            {
                if (File.Exists($"{Application.persistentDataPath}/cck-update-0.unitypackage")) File.Delete($"{Application.persistentDataPath}/cck-update-0.unitypackage");
                uwr.downloadHandler = new DownloadHandlerFile($"{Application.persistentDataPath}/cck-update-0.unitypackage");
                await uwr.SendWebRequest();
            }
            Directory.Delete("Assets/ABI.CCK", true);
            AssetDatabase.ImportPackage($"{Application.persistentDataPath}/cck-update-0.unitypackage", false);
        }
    }

    [Serializable]
    public class CCKVersionCurrent
    {
        public int BuildID { get; set; }
        public string BuildVersion { get; set; }
        public string DownloadLink { get; set; }
    }
}